// This file is part of Dagrs.

// Dagrs is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// Dagrs is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with Dagrs; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


import java.util.*;
import java.awt.*;

public class Algorithm extends Thread{
    private Topology t;
    private Vector rules;
    private Random r;
    private Hashtable defState;
    private Hashtable distState;
    private int tempo;
    
    public Algorithm(){
        this.rules = new Vector();
        this.r = new Random();
        this.defState = new Hashtable(); // Default state at init
        this.distState = new Hashtable(); // One distinguished state at init
    }
    public void setTopology(Topology t){
        this.t = t;
    }
    public Topology getTopology(){
        return this.t;
    }
    public void addDefState(String state){
        StringTokenizer st = new StringTokenizer(state,",");
        defState.put(st.nextToken(),st.nextToken());
    }
    public void addDistState(String state){
        StringTokenizer st = new StringTokenizer(state,",");
        distState.put(st.nextToken(),st.nextToken());
    }
    public void setDefState(Hashtable def){
        this.defState=def;
    }
    public Hashtable getDefState(){
        return defState;
    }
    public void setTempo(int t){
        this.tempo=t;
    }
    public int getTempo(){
        return this.tempo;
    }
    public void setDistState(Hashtable dist){
        this.distState=dist;
    }
    public Hashtable getDistState(){
        return distState;
    }
    public void addRule(Rule r){
        rules.add(r);
    }
    public Vector getRules(){
        return this.rules;
    }
    public void setRules(Vector rules){
        this.rules = rules;
    }
    public void run(){
        Vertex v1;
        Neighbor n2;
        Vector vertices=t.getVertices();
        while(true){
            try{
                while(vertices.size()<1);
                //Tire un noeud au hasard
                v1=(Vertex)vertices.elementAt(r.nextInt(vertices.size()));
                //Met a jour sa liste de voisins
                t.updateNeighbors(v1);
                //Tire au hasard un de ses voisins
                int nbn=v1.neighbors.size();
                if (nbn>0){
                    int num=r.nextInt(v1.neighbors.size());
                    n2=(Neighbor)v1.neighbors.elementAt(num);
                    //Met a jour sa liste de voisins
                    t.updateNeighbors(n2.getVertex()); // NON OPTIMISE
                    executeOneRule(v1,n2);
                }
            }catch(Exception e){
                e.printStackTrace();
            }
            
	    try{
                if(tempo!=0){
                    Thread.sleep(tempo);
                }
            }catch(Exception e){
                e.printStackTrace();
            }
        }
    }
    private boolean executeOneRule(Vertex v1, Neighbor n2){
        for (int i=0; i<rules.size(); i++)
            if (((Rule)rules.elementAt(i)).execute(v1,n2)) return true;
        return false;
    }
}
