// This file is part of Dagrs.

// Dagrs is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// Dagrs is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with Dagrs; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.awt.*;
import java.util.*;
import java.io.File;

public class Command extends JPanel implements ActionListener, ChangeListener{
    Topology t;
    TopologyRandomizer r;
    Editor ed;
    Algorithm algo;
    JButton rand;
    JButton snap;
    JSlider range;
    JSlider tempo;
    int TEMPO_MAX=1000;

    public Command(Topology t, Algorithm algo, Editor ed){
        this.t=t;
        this.algo=algo;
        this.ed=ed;
        this.createComponents();
    }
    public void createComponents(){
        createMenus();
        rand = new JButton("start Rando");
        rand.addActionListener(this);
        add(rand);
        snap = new JButton("Evolve step");
        snap.addActionListener(this);
        add(snap);
        range = new JSlider(JSlider.HORIZONTAL,1,500,(int)t.getRange());
        range.addChangeListener(this);
        Panel tmp=new Panel(new GridLayout(2,1));
        tmp.add(new JLabel("  Radio range:"));
        tmp.add(range);
        add(tmp);
        tempo = new JSlider(JSlider.HORIZONTAL,0,TEMPO_MAX,0);
        tempo.addChangeListener(this);
        tmp=new Panel(new GridLayout(2,1));
        tmp.add(new JLabel("Computation speed:"));
        tmp.add(tempo);
        add(tmp);
    }
    private void createMenus(){
        JMenuBar jmb = new JMenuBar();
        JMenu jmFile = new JMenu("File");
        jmFile.add(createJMenuItem("New Algorithm"));
        jmFile.add(createJMenuItem("Open Algorithm"));
        jmFile.add(createJMenuItem("Save Algorithm"));
        jmFile.add(new JSeparator());
        jmFile.add(createJMenuItem("Export physic graph"));
        jmFile.add(createJMenuItem("Export logic graph"));
        jmFile.add(createJMenuItem("Export evolving graph"));
        jmb.add(jmFile);
        add(jmb);        
    }
    private JMenuItem createJMenuItem(String s){
        JMenuItem tmp = new JMenuItem(s);
        tmp.addActionListener(this);
        return tmp;
    }
    public void actionPerformed(ActionEvent e){
        String action=((AbstractButton)e.getSource()).getText();
        // Algorithm Opening
        if (action.compareTo("New Algorithm")==0){
            algo.stop();
            algo=new Algorithm();
            algo.setTopology(t);
            ed.setAlgorithm(algo);
            this.algo=algo;
            algo.start();
        }
        else if (action.compareTo("Open Algorithm")==0){
            JFileChooser jfc = new JFileChooser();
            jfc.setCurrentDirectory(new File("./algos/"));
            int returnVal = jfc.showOpenDialog(this);
            if(returnVal == JFileChooser.APPROVE_OPTION) {
                String s=jfc.getSelectedFile().getAbsolutePath();
                algo.stop();
                algo=AlgoReader.getAlgoFromFile(s);
                algo.setTopology(t);
                ed.setAlgorithm(algo);
                this.algo=algo;
                algo.start();
            }
        }
        // Algorithm Saving
        else if (action.compareTo("Save Algorithm")==0){
            JFileChooser jfc = new JFileChooser();
            jfc.setCurrentDirectory(new File("."));
            int returnVal = jfc.showSaveDialog(this);
            if(returnVal == JFileChooser.APPROVE_OPTION) {
                String s=jfc.getSelectedFile().getAbsolutePath();
                AlgoWriter.putAlgoToFile(ed.getAlgorithm(), s);
            }
        }
        // Exporting physic graph to PS
        else if (action.compareTo("Export physic graph")==0){
            TopologyToFiles.toFiles(t, "./export/physic", 
                                    TopologyToFiles.PHYSIC);
        }
        // Exporting logic graph to PS
        else if (action.compareTo("Export logic graph")==0){
            TopologyToFiles.toFiles(t, "./export/logic", 
                                    TopologyToFiles.LOGIC);
        }
        else if (action.compareTo("Export evolving graph")==0){
            TopologyToFiles.toFiles(t, "./export/evolving", 
                                    TopologyToFiles.EVOLVING);
        }
        // Start Randomizing
        else if (action.compareTo("start Rando")==0){
            r = new TopologyRandomizer(t, 10, 500, 500);
            r.start();
            rand.setText("stop Rando");
        }
        // Stop Randomizing
        else if (action.compareTo("stop Rando")==0){
            try{
                r.stop();
            }catch (Exception ex){}
            rand.setText("start Rando");
        }
        // Take Snapshot
        else if (action.compareTo("Evolve step")==0){
            t.snapEvolveStep();
        }
    }
    public void stateChanged(ChangeEvent event){
        if (event.getSource()==(Object)range){
            t.setRange((double)range.getValue());
        } else if (event.getSource()==(Object)tempo){
            int t=(int)tempo.getValue();
            if (t==TEMPO_MAX)
                algo.setTempo(30000);
            else
                algo.setTempo(t);
        }
    }
}

