// This file is part of Dagrs.

// Dagrs is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// Dagrs is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with Dagrs; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


import java.util.Vector;

public class Condition{
    int v_left, v_right, v_mod=0; 
    String l_left, l_right, l_mod;
    String val_right;
    char operator;
    int modifier=0;
    /* A condition can be as follow :
       v_left.l_left {=|!|<|>} {val_right | v_right.l_right [+ {v_mod.l_mod | modifier}]}
    */
    public Condition(String cond){
        int ind=3;
        int deb;
        int taille=cond.length();
        int tmp;
        // v_left
        v_left=new Integer(cond.substring(1,2)).intValue();
        // l_left
        char c=cond.charAt(ind);
        while (c!=' ' && c!='=' && c!='!' && c!='<' && c!='>'){
            ind++;
            c=cond.charAt(ind);
        }
        l_left=cond.substring(3,ind);
        // operator
        while (cond.charAt(ind)==' '){ind++;}
        operator=cond.charAt(ind);
        do{ind++;}while(cond.charAt(ind)==' ');

        // v_right or val_right
        tmp=cond.indexOf('.', ind);
        if (tmp!=-1){
            // v_right
            v_right=new Integer(cond.substring(ind+1, ind+2)).intValue();
            // l_right
            ind=ind+3;tmp++;
            c=cond.charAt(ind);
            while(c!=' ' && c!='+' && tmp!=taille)
                c=cond.charAt(tmp++);
            if (tmp==taille)
                l_right=cond.substring(ind,tmp);
            else
                l_right=cond.substring(ind,tmp-1);
            // modifier

            tmp=cond.indexOf('+');
            if (tmp != -1){
                String strmod=cond.substring(tmp+1,taille).trim();
                try{
                    modifier=new Integer(strmod).intValue();
                }catch(Exception e){
                    v_mod=Integer.parseInt(strmod.substring(1,2));
                    l_mod=strmod.substring(3,strmod.length());
                }
            }
        }else{
            val_right=cond.substring(ind,taille).trim();
        }
    }
    public boolean verify(Vertex v1, Neighbor n2){
        if (val_right!=null)
            return compare(getValue(v1,n2,v_left,l_left),val_right,operator);
        else{
            String gauche=getValue(v1,n2,v_left,l_left);
            String droite=getValue(v1,n2,v_right,l_right);
            if (modifier!=0){
                return compare(new Integer(gauche),new Integer(Integer.parseInt(droite) + modifier), operator);
            }else if(v_mod!=0){
                int mod=Integer.parseInt(getValue(v1,n2,v_mod,l_mod));
                return compare(new Integer(gauche),new Integer(Integer.parseInt(droite) + mod), operator);
            }else{
                return compare(gauche, droite, operator);
            }
        }
    }
    public void execute(Vertex v1, Neighbor n2){
        if (val_right!=null){
            setValue(v1,n2,v_left,l_left,val_right);
            return;
        }else{
            //String gauche=getValue(v1,n2,v_left,l_left);
            String droite=getValue(v1,n2,v_right,l_right);
            if (modifier!=0)
                droite=new Integer(Integer.parseInt(droite) + modifier).toString();
            if (v_mod!=0){
                int mod=Integer.parseInt(getValue(v1,n2,v_mod,l_mod));
                droite=new Integer(Integer.parseInt(droite) + mod).toString();
            }
            setValue(v1,n2,v_left,l_left,droite);
        }
    }
    // get the value of label l of vertex v
    private static String getValue(Vertex v1, Neighbor n2, int v, String l){
        // Cas special necessitant pas mal de traitements :
        if (l.compareTo("edgelabel")==0 && v==2)
            return getEdgeLabelFromNeighbor(v1,n2);

        if (l.compareTo("edgelabel")==0)
            return n2.getLinkLabel();
        else if (l.compareTo("edgestate")==0)
            return n2.getLinkState();

        // Cas normal
        if (v==1)
            return v1.getLabel(l);
        else
            return n2.getVertex().getLabel(l);
    }
    // set the value of label l of vertex v
    private static void setValue(Vertex v1, Neighbor n2, int v, String l, String val){
        // Cas special necessitant pas mal de traitements :
        if (l.compareTo("edgelabel")==0 && v==2){
            setEdgeLabelFromNeighbor(v1,n2,val);
            return;
        }

        if (l.compareTo("edgelabel")==0){
            n2.setLinkLabel(val);
            return;
        }

        // Cas normal
        if (v==1)
            v1.setLabel(l,val);
        else
            n2.getVertex().setLabel(l,val);
    }
    // return the value of the label of the edge (v1, n2) from the neighbor's side
    private static String getEdgeLabelFromNeighbor(Vertex v1, Neighbor n2){
        Vector vec = n2.getVertex().neighbors;
        Neighbor tmp;
        int i=0;
        while (((tmp=(Neighbor)vec.elementAt(i++)).getVertex()) != v1);
        return tmp.getLinkLabel();
    }
    // return the value of the label of the edge (v1, n2) from the neighbor's side
    private static void setEdgeLabelFromNeighbor(Vertex v1, Neighbor n2, String val){
        Vector vec = n2.getVertex().neighbors;
        Neighbor tmp;
        int i=0;
        while (((tmp=(Neighbor)vec.elementAt(i++)).getVertex()) != v1);
        tmp.setLinkLabel(val);
    }
    private static boolean compare(Comparable c1, Comparable c2, char operator){
        switch (operator){
        case '=':
            return (c1.compareTo(c2)==0);
        case '!':
            return (c1.compareTo(c2)!=0);
        case '<':
            return (c1.compareTo(c2)<0);
        default:
            return (c1.compareTo(c2)>0);
        }
    }
    public String toString(){
        String s="";
        if (val_right!=null){
            //System.out.println("1");
            s="v" + v_left + "." + l_left + " " + operator + " " + val_right;
        }else
            s="v" + v_left + "." + l_left + " " + operator + " v" +  v_right + "." + l_right;
            if (modifier!=0){
                //System.out.println("2");
                s=s + " + " + modifier;
            }else if(v_mod!=0){
                //System.out.println("3");
                s=s + " + v" + v_mod + "." + l_mod;
            }
            //System.out.println(s);
        return s;
    }        
}
