// This file is part of Dagrs.

// Dagrs is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// Dagrs is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with Dagrs; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


import javax.swing.*;
import javax.swing.text.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class Editor extends JTabbedPane implements ActionListener{
    Algorithm algo;
    // Rules Panel
    JPanel panRules;
    Vector preconds;
    Vector actions;
    // States Panel
    JPanel panStates;
    Vector defState;
    Vector distState;
    JTextField newDefLabel;
    JTextField newDistLabel;
    // Action Panel
    JPanel panAction;
    Vertex selected;
    JTextField action;

    public Editor(){
        this.setBackground(new Color(250,250,250));
        this.setPreferredSize(new Dimension(300,500));
        add("States", panStates=new JPanel(new FlowLayout(FlowLayout.RIGHT)));
        add("Rules", panRules=new JPanel(new FlowLayout(FlowLayout.RIGHT)));
        add("Action", panAction=new JPanel(new FlowLayout(FlowLayout.RIGHT)));
        newDefLabel = new JTextField(8);
        newDistLabel = new JTextField(8);
        action = new JTextField(20);
    }
    public void setSelectedVertex(Vertex v){
        this.selected = v;
        drawPanAction();
    }
    public void setAlgorithm(Algorithm algo){
        this.algo=algo;
        this.preconds = new Vector();
        this.actions = new Vector();
        this.defState = new Vector();
        this.distState = new Vector();
        // panStates
        Hashtable hdef=algo.getDefState();
        for (Enumeration e = hdef.keys(); e.hasMoreElements() ;){
            String key=(String)e.nextElement();
            JTextField value=new JTextField(10);
            value.setText((String)hdef.get(key));
            defState.add(new JLabel(key));
            defState.add(value);
        }
        Hashtable hdist=algo.getDistState();
        for (Enumeration e = hdist.keys(); e.hasMoreElements() ;){
            String key=(String)e.nextElement();
            JTextField value=new JTextField(10);
            value.setText((String)hdist.get(key));
            distState.add(new JLabel(key));
            distState.add(value);
        }
        Topology t = algo.getTopology();
        t.removeAll();
        t.setDefaultLabel(algo.getDefState());
        if (distState.size()>0)
            t.addVertex(algo.getDistState(),100,100);

        // panRules
        Vector tmp=algo.getRules();
        for (int i=0; i<tmp.size(); i++){
            Rule r=(Rule)tmp.elementAt(i);
            JTextField txtpre = new JTextField(20);
            txtpre.setText(r.getPreconds().toString());
            JTextField txtact = new JTextField(20);
            txtact.setText(r.getActions().toString());
            preconds.add(txtpre);
            actions.add(txtact);
        }
        drawPanStates();
        drawPanRules();
        drawPanAction();
    }
    public Algorithm getAlgorithm(){
        return algo;
    }
    public void drawPanStates(){
        panStates.removeAll();
        String delim="----------------------------------------";
        panStates.add(new JLabel(delim + "Default"));
        for (int i=0; i<defState.size(); i++){
            panStates.add(new JLabel("                       "));
            panStates.add((JLabel)defState.elementAt(i++));
            panStates.add((JTextField)defState.elementAt(i));
        }
        panStates.add(new JLabel("new label:"));
        panStates.add(newDefLabel);
        JButton jbdef=new JButton("Add def");
        jbdef.addActionListener(this);
        panStates.add(jbdef);
        panStates.add(new JLabel(delim + "Distinguished"));
        for (int i=0; i<distState.size(); i++){
            panStates.add(new JLabel("                       "));
            panStates.add((JLabel)distState.elementAt(i++));
            panStates.add((JTextField)distState.elementAt(i));
        }
        panStates.add(new JLabel("new label:"));
        panStates.add(newDistLabel);
        JButton jbdist=new JButton("Add dist");
        jbdist.addActionListener(this);
        panStates.add(jbdist);
        panStates.add(new JLabel(delim));
        JButton commit=new JButton("Commit states");
        commit.addActionListener(this);
        panStates.add(commit);
        panStates.updateUI();
    }
    public void drawPanRules(){
        panRules.removeAll();
        String delim="-----------------------------------------Rule ";
        for (int i=0; i<preconds.size(); i++){
            panRules.add(new JLabel(delim+i));
            panRules.add(new JLabel("preconds"));
            panRules.add((JTextField)preconds.elementAt(i));
            panRules.add(new JLabel("actions"));
            panRules.add((JTextField)actions.elementAt(i));
        }
        JButton add=new JButton("Add rule");
        add.addActionListener(this);
        panRules.add(add);
        JButton commit=new JButton("Commit rules");
        commit.addActionListener(this);
        panRules.add(commit);
        panRules.updateUI();
    }
    public void drawPanAction(){
        panAction.removeAll();
        if (selected==null){
            panAction.add(new JLabel("Please select a vertex (middle click)"));
        }else{
            String strtmp;
            Vector neighbors=selected.neighbors;
            Enumeration keys = selected.getAllLabels().keys();
            Enumeration vals = selected.getAllLabels().elements();
            for (; keys.hasMoreElements() ;) {
                strtmp = "                                                  ";
                strtmp = strtmp + (String)keys.nextElement();
                strtmp = strtmp + " : " + (String)vals.nextElement();
                JLabel tmp=new JLabel(strtmp);
                panAction.add(tmp);
            }          
            for (int i=0; i<neighbors.size();i++){
                strtmp = new String();
                strtmp = "neighbor " + i + " : " + ((Neighbor)neighbors.elementAt(i)).getLinkLabel();
                strtmp = strtmp + " - " + ((Neighbor)neighbors.elementAt(i)).getLinkState();
                strtmp = "                                                                     " + strtmp;
                JLabel tmp=new JLabel(strtmp);
                panAction.add(tmp);
            }
            panAction.add(new Label("Action"));
            panAction.add(action);
            JButton jb=new JButton("Execute");
            jb.addActionListener(this);
            panAction.add(jb);
        }
        panAction.updateUI();
    }
    public void actionPerformed(ActionEvent e){
        String name=((AbstractButton)e.getSource()).getText();
        // Algorithm Opening
        if (name.compareTo("Add rule")==0){
            preconds.add(new JTextField(20));
            actions.add(new JTextField(20));
            drawPanRules();
        }
        else if (name.compareTo("Commit rules")==0){
            boolean isAlive=algo.isAlive();
            if (isAlive)
                algo.stop();
            algo=generateAlgorithmFromEditor();
            drawPanRules();
            if (isAlive)
                algo.start();
        }
        else if (name.compareTo("Add def")==0){
            defState.add(new JLabel(newDefLabel.getText()));
            newDefLabel.setText("");
            defState.add(new JTextField(10));
            drawPanStates();
        }
        else if (name.compareTo("Add dist")==0){
            distState.add(new JLabel(newDistLabel.getText()));
            newDistLabel.setText("");
            distState.add(new JTextField(10));
            drawPanStates();
        }
        else if (name.compareTo("Commit states")==0){
            boolean isAlive=algo.isAlive();
            if (isAlive)
                algo.stop();
            algo=generateAlgorithmFromEditor();
            Topology t=algo.getTopology();
            t.removeAll();
            t.setDefaultLabel(algo.getDefState());
            if (distState.size()>0)
                t.addVertex(algo.getDistState(),100,100);
            drawPanStates();
            if (isAlive)
                algo.start();
        }else if (name.compareTo("Execute")==0){
            new TreeNode(action.getText()).execute(selected,null);
            selected=null;
            drawPanAction();
        }
    }

    public Algorithm generateAlgorithmFromEditor(){
        Algorithm algotmp = new Algorithm();
        algotmp.setTopology(algo.getTopology());

        // States
        Hashtable hdef=new Hashtable();
        for (int i=0; i<defState.size(); i++){
            JLabel jl=(JLabel)defState.elementAt(i++);
            String key=jl.getText();
            JTextField jtf=(JTextField)defState.elementAt(i);
            String value=jtf.getText();
            if (key.compareTo("")!=0 && value.compareTo("")!=0)
                hdef.put(key,value);
            else{
                defState.remove(jl);
                defState.remove(jtf);
                i-=2;
            }
        }
        algotmp.setDefState(hdef);
        Hashtable hdist=new Hashtable();
        for (int i=0; i<distState.size(); i++){
            JLabel jl=(JLabel)distState.elementAt(i++);
            String key=jl.getText();
            JTextField jtf=(JTextField)distState.elementAt(i);
            String value=jtf.getText();
            if (key.compareTo("")!=0 && value.compareTo("")!=0)
                hdist.put(key,value);
            else{
                distState.remove(jl);
                distState.remove(jtf);
                i-=2;
            }
        }
        algotmp.setDistState(hdist);
        
        // Rules
        for (int i=0; i<preconds.size();i++){
            JTextField pre = (JTextField)preconds.elementAt(i);
            JTextField act = (JTextField)actions.elementAt(i);
            if (pre.getText().compareTo("")!=0 && act.getText().compareTo("")!=0){
                Rule r=new Rule();
                r.setPreconds(pre.getText());
                r.setActions(act.getText());
                algotmp.addRule(r);
            }else{
                preconds.remove(pre);
                actions.remove(act);
                i--;
            }
        }
        return algotmp;
    }
}
