import java.util.*;
import java.io.*;

public class EvolvingGraph{
    private Vector nodes;
    private Vector edges;
    private boolean hasCoords=false;
    private Integer maxTime=new Integer(0);

    public EvolvingGraph(){
        nodes=new Vector();
        edges=new Vector();
    }
    public EvolvingGraph(String filename){
        this();
        try{
            BufferedReader in=new BufferedReader(new FileReader(filename));
            StringTokenizer st;
            String sin="";
            do sin=in.readLine(); while (sin.compareTo("vertices")!=0);
            st = new StringTokenizer(sin," ");
            while ((sin=in.readLine()).compareTo("")!=0){
                st = new StringTokenizer(sin," ");
                String id=st.nextToken();
                if (st.hasMoreTokens()){
                    int x=Integer.parseInt(st.nextToken());
                    int y=Integer.parseInt(st.nextToken());
                    addNode(id, x, y);
                }else{
                    addNode(id);
                }
            }

            do sin=in.readLine(); while (sin.compareTo("edges")!=0);
            while ((sin=in.readLine()).compareTo("")!=0){
                st = new StringTokenizer(sin," ");
                String times="";
                String n1=st.nextToken();
                String n2=st.nextToken();
                
                // Get the label
                if (st.hasMoreTokens())
                    times=st.nextToken();
                while (st.hasMoreTokens())
                    times=times+" "+st.nextToken();

                addEdge(n1,n2,times);
            }
        }catch(Exception e){
        }        
    }

    class Node{
        public String id;
        public int x=0,y=0;
        public Node(String id){
            this.id=id.toString();
        }
        public Node(String id, int x, int y){
            this(id);
            this.x=x;
            this.y=y;
        }
    }
    class Edge{
        public String n1;
        public String n2;
        public Vector times;

        public Edge(String n1, String n2){
            this.n1=n1.toString();
            this.n2=n2.toString();
            this.times=new Vector();
        }
        public Edge(String n1, String n2, Integer time){
            this(n1, n2);
            this.times.add(time);
        }
        public Edge(String n1, String n2, Vector times){
            this(n1, n2);
            this.times=times;
        }
        public boolean equal(String n1, String n2){
            return ((this.n1.compareTo(n1)==0 && this.n2.compareTo(n2)==0) ||
                    (this.n1.compareTo(n2)==0 && this.n2.compareTo(n1)==0));
        }
    }

    public void addNode(String id){
        boolean exists=false;
        for (int i=0; i<nodes.size(); i++)
            if (((Node)nodes.elementAt(i)).id.compareTo(id)==0)
                exists=true;
        if (!exists)
            nodes.add(new Node(id));
    }
    public void addNode(String id, int x, int y){
        Node n;
        boolean exists=false;
        for (int i=0; i<nodes.size(); i++){
            n=(Node)nodes.elementAt(i);
            if (n.id.compareTo(id)==0){
                exists=true;
                n.x=x;
                n.y=y;
            }
        }
        if (!exists)
            nodes.add(new Node(id,x,y));
        if (x!=0 || y!=0)
            hasCoords=true;
    }
    public void addEdge(String n1, String n2, Integer time){
        Edge e;
        boolean exists=false;
        for (int i=0; i<edges.size(); i++){
            e=(Edge)edges.elementAt(i);
            if (e.equal(n1,n2)){
                exists=true;
                if (((Integer)e.times.elementAt(e.times.size()-1))
                    .compareTo(time)!=0)
                    e.times.add(time);
            }
        }
        if (!exists)
            edges.add(new Edge(n1,n2,time));
        if (time.compareTo(maxTime)>0) maxTime=time;
    }
    public void addEdge(String n1, String n2, String times){
        Edge e;
        boolean exists=false;
        Vector v=intervalFromString(times);
        for (int i=0; i<edges.size(); i++){
            e=(Edge)edges.elementAt(i);
            if (e.equal(n1,n2)){
                exists=true;
                e.times=v;
            }
        }
        if (!exists)
            edges.add(new Edge(n1,n2,v));
        if (((Integer)v.elementAt(v.size()-1)).compareTo(maxTime)>0)
            maxTime=(Integer)v.elementAt(v.size()-1);
    }
    public static String intervalToString(Vector vtimes){
        String stimes=" ";
        for (int i=0; i<vtimes.size()-1; i++){
            Integer t=(Integer)vtimes.elementAt(i);
            char last=stimes.charAt(stimes.length()-1);
            if (last==' '){
                stimes=stimes+t.toString();
                if (((Integer)vtimes.elementAt(i+1)).compareTo(new Integer(t.intValue()+1))==0)
                    stimes=stimes+"-";
                else
                    stimes=stimes+" ";
            }else{
                if (((Integer)vtimes.elementAt(i+1)).compareTo(new Integer(t.intValue()+1))!=0)
                    stimes=stimes+t.toString()+" ";
            }
        }
        stimes=stimes+vtimes.elementAt(vtimes.size()-1);
        stimes=stimes.substring(1, stimes.length());
        return stimes;
    }
    private static String getHVal(Hashtable h, Integer i){
        if (h.containsKey(i))
            return (String)h.get(i);
        else
            return "";
    }
    private void fillEdgeEvents(Edge e, Hashtable h){
        String stimes=intervalToString(e.times);
        StringTokenizer st=new StringTokenizer(stimes);
        while (st.hasMoreElements()){
            String tmp=st.nextToken();
            int ind=tmp.indexOf('-');
            Integer beg, end;
            if (ind==-1){
                beg=new Integer(Integer.parseInt(tmp)-1);
                end=new Integer(tmp);
            }else{
                beg=new Integer(Integer.parseInt(tmp.substring(0,ind))-1);
                end=new Integer(tmp.substring(ind+1,tmp.length()));
            }
            h.put(beg, getHVal(h, beg)+"ae \""+
                  e.n1+e.n2+"\" \""+e.n1+"\" \""+e.n2+"\"\n");
            if (end.compareTo(maxTime)!=0)
                h.put(end, getHVal(h, end)+"de \""+e.n1+e.n2+"\"\n");
        }
    }
    public static Vector intervalFromString(String stimes){
        StringTokenizer st=new StringTokenizer(stimes);
        Vector vtimes=new Vector();
        while (st.hasMoreElements()){
            String tmp=st.nextToken();
            int ind=tmp.indexOf('-');
            if (ind==-1){
                vtimes.add(new Integer(tmp));
            }else{
                int beg=Integer.parseInt(tmp.substring(0,ind));
                int end=Integer.parseInt(tmp.substring(ind+1,tmp.length()));
                for (int i=beg; i<=end; i++){
                    vtimes.add(new Integer(i));
                }
            }
        }
        return vtimes;
    }
    public boolean hasCoords(){
        return hasCoords;
    }
    public String getNodesEvents(){ //DGS Format
        String s="";
        for (int i=0; i<nodes.size(); i++){
            Node n=(Node)nodes.elementAt(i);
            s=s+"an"+" \""+n.id+"\" "+n.x+" "+n.y+"\n";
        }
        return s;
    }
    public Vector getEdgesEvents(){ //DGS Format
        Hashtable h=new Hashtable();
        for (int i=0; i<edges.size(); i++){
            Edge e=(Edge)edges.elementAt(i);
            fillEdgeEvents(e,h);
        }
        Vector v=new Vector();
        v.setSize(maxTime.intValue());
        for (int i=0; i<maxTime.intValue(); i++){
            Integer I=new Integer(i);
            if (h.containsKey(I))
                v.setElementAt(h.get(I),i);
        }
        return v;
    }
    public String toString(){
        String sv="", se="";
        int xmin=10000,ymin=10000,xmax=0,ymax=0;
        int id;
        for (int i=0; i<nodes.size(); i++){
            Node n=(Node)nodes.elementAt(i);
            if(n.x > xmax) xmax = n.x;
            if(n.y > ymax) ymax = n.y;
            if(n.x < xmin) xmin = n.x;
            if(n.y < ymin) ymin = n.y;
        }
        for (int i=0; i<nodes.size(); i++){
            Node n=(Node)nodes.elementAt(i);
            sv=sv+n.id+" "+(n.x-xmin)+" "+(n.y-ymin)+"\n";
        }
        for (int i=0; i<edges.size(); i++){
            Edge e=(Edge)edges.elementAt(i);
            se=se+e.n1+" "+e.n2+" "+intervalToString(e.times)+"\n";
        }
        String sf="dimensions\n";
        sf=sf+(xmax-xmin)+" "+(ymax-ymin)+"\n\n";
        sf=sf+"vertices\n";
        sf=sf+sv+"\n";
        sf=sf+"edges\n";
        sf=sf+se+"\n";
        return sf;
    }
}
