import java.io.*;
import java.util.StringTokenizer;
import java.util.Hashtable;
import java.awt.Point;

public class GraphToEPS{
    public static final int LABELLED=0, COLORED=1;

    // Generate the headers of the EPS File
    private static String getHeaders(double dimx, double dimy){
	String s = new String();
	s=s.concat("%!PS-Adobe-2.0 EPSF-2.0\n");
	s=s.concat("%%Creator: DA-GRS Simulator\n");
	s=s.concat("%%BoundingBox: 0 0 "+ dimx +" "+ dimy +"\n");
	s=s.concat("/$F2psDict 200 dict def\n");
	s=s.concat("$F2psDict begin\n");
	s=s.concat("$F2psDict /mtrx matrix put\n");
	s=s.concat("/col0 {0.000 0.000 0.000 srgb} bind def\n");
	s=s.concat("\n");
	s=s.concat("end\n");
	s=s.concat("save\n");
	s=s.concat("0.0 "+dimy+" translate\n");
	s=s.concat("1 -1 scale\n");
	s=s.concat("\n");
	s=s.concat("/ef {eofill} bind def\n");
	s=s.concat("/gr {grestore} bind def\n");
	s=s.concat("/gs {gsave} bind def\n");
        s=s.concat("/ff {findfont} bind def\n");
        s=s.concat("/sf {setfont} bind def\n");
        s=s.concat("/scf {scalefont} bind def\n");
        s=s.concat("/sh {show} bind def\n");
	s=s.concat("/l {lineto} bind def\n");
	s=s.concat("/m {moveto} bind def\n");
	s=s.concat("/n {newpath} bind def\n");
	s=s.concat("/s {stroke} bind def\n");
	s=s.concat("/srgb {setrgbcolor} bind def\n");
        s=s.concat("/slw {setlinewidth} bind def\n");
	s=s.concat("/sc {scale} bind def\n");
	s=s.concat("/tr {translate} bind def\n");
	s=s.concat(" /DrawEllipse {\n");
	s=s.concat("	/endangle exch def\n");
	s=s.concat("	/startangle exch def\n");
	s=s.concat("	/yrad exch def\n");
	s=s.concat("	/xrad exch def\n");
	s=s.concat("	/y exch def\n");
	s=s.concat("	/x exch def\n");
	s=s.concat("	/savematrix mtrx currentmatrix def\n");
	s=s.concat("	x y tr xrad yrad sc 0 0 1 startangle endangle arc\n");
	s=s.concat("	closepath\n");
	s=s.concat("	savematrix setmatrix\n");
	s=s.concat("	} def\n");
	s=s.concat("\n");
	s=s.concat("/$F2psBegin {$F2psDict begin /$F2psEnteredState save def} def\n");
	s=s.concat("/$F2psEnd {$F2psEnteredState restore end} def\n");
	s=s.concat("\n");
	s=s.concat("$F2psBegin\n");
	s=s.concat("10 setmiterlimit\n");
	//s=s.concat(" 0.06299 0.06299 sc\n");
	s=s.concat(" 0.8 0.8 sc\n");
	s=s.concat("\n");
        return s;
    }

    // Generate the footers of the EPS file
    private static String getFooters(){
	String s = new String();
	s=s.concat("% here ends figure;\n");
	s=s.concat("$F2psEnd\n");
	s=s.concat("restore\n");
	s=s.concat("showpage\n");
	s=s.concat("%%Trailer\n");
	s=s.concat("%EOF\n");
	return s;
    }

    // Convert the input format to EPS
    private static void convert(BufferedReader in, PrintWriter out, int mode) 
        throws Exception{
        String sin="", sout="";
        double scale=2;
        
        // headers
        int dimx, dimy;
        do sin=in.readLine(); while (sin.compareTo("dimensions")!=0);
        sin=in.readLine();
        StringTokenizer st = new StringTokenizer(sin," ");
        dimx=Integer.parseInt(st.nextToken());
        dimy=Integer.parseInt(st.nextToken());
        out.println(getHeaders(((double)dimx)*scale, ((double)dimy)*scale));
        
        // vertices
        do sin=in.readLine(); while (sin.compareTo("vertices")!=0);
        st = new StringTokenizer(sin," ");
        Hashtable vertices=new Hashtable();
        while ((sin=in.readLine()).compareTo("")!=0){
            int x, y;
            st = new StringTokenizer(sin," ");
            String id=st.nextToken();
            x=(int)(((double)(Integer.parseInt(st.nextToken())+10))*scale);
            y=(int)(((double)(Integer.parseInt(st.nextToken())+10))*scale);
            vertices.put(id,new Point(x,y));
            // write the EPS lines
            out.print("n "+ x +" "+ y +" 8 8");
	    out.print(" 0 360 ");
	    out.println("DrawEllipse gs 0.00 setgray ef gr gs col0 s gr");

            out.print("/Times-Roman ff 15 scf sf\n");
            out.print(x + " " + y + " m\n");
            if(mode==2){
                out.println("gs 1 -1 sc (  "+
                            id.substring(id.length()-3,id.length())+
                            ") col0 sh gr\n");
            }
        }

        // edges
        do sin=in.readLine(); while (sin.compareTo("edges")!=0);
        while ((sin=in.readLine()).compareTo("")!=0){
            String label="";
            Point p1, p2;
            st = new StringTokenizer(sin," ");
            p1=((Point)vertices.get(st.nextToken()));
            p2=((Point)vertices.get(st.nextToken()));
            
            // Get the label
            if (st.hasMoreTokens())
                label=st.nextToken();
            while (st.hasMoreTokens())
                label=label+" "+st.nextToken();
            
            int x1, y1, x2, y2;
            x1=(int)p1.getX();
            y1=(int)p1.getY();
            x2=(int)p2.getX();
            y2=(int)p2.getY();
            out.print("n " + x1 + " " + y1 + " ");
            out.print("m " + x2 + " " + y2 + " ");
            if (label=="" || label.compareTo("0")==0){
                out.println("l 0.6 slw gs col0 s gr");
            }else{
                if (mode==COLORED){
                    out.println("l 0 0 0.5 srgb 3 slw gs s gr");
                }else{ // LABELED
                    int xe, ye;
                    xe=(x1 + x2)/2;
                    ye=(y1 + y2)/2;
                    out.print("l 0.6 slw gs col0 s gr\n");
                    out.print("/Times-Roman ff 15 scf sf\n");
                    out.print(xe + " " + ye + " m\n");
                    out.println("gs 1 -1 sc ("+label+") col0 sh gr\n");
                }
            }
        }
                

        // footers
        out.println(getFooters());
    }
    public static void main(String args[]){
        main(args[0], args[1], Integer.parseInt(args[2]));
    }
    public static void main(String fin, String fout, int mode){
        try{
            BufferedReader reader = 
                new BufferedReader(new FileReader(fin));
            PrintWriter writer = 
                new PrintWriter(new BufferedWriter(new FileWriter(fout)));
            convert (reader, writer, mode);
            reader.close();
            writer.close();
        }catch(Exception e){
            e.printStackTrace();
        }        
    }
}
