// This file is part of Dagrs.

// Dagrs is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// Dagrs is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with Dagrs; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

public class JTopology extends JPanel implements MouseListener{

    private Vector jvertices; // Les composants graphiques des noeuds
    private Topology t;
    private Thread thread;
    private Hashtable colors;
    private Editor ed;

    public JTopology(Topology t, final int delay){
        jvertices = new Vector();
        this.t=t;
        this.setLayout(null);
        this.setBackground(new Color(150,150,150));
        this.setPreferredSize(new Dimension(600,500));
        this.addMouseListener(this);
        // colors creation
        colors=new Hashtable();
        colors.put("white", Color.white);
        colors.put("yellow", Color.yellow);
        colors.put("pink", Color.pink);
        colors.put("magenta", Color.magenta);
        colors.put("orange", Color.orange);
        colors.put("cyan", Color.cyan);
        colors.put("red", Color.red);
        colors.put("blue", Color.blue);
        colors.put("green", Color.green);
        colors.put("gray", Color.darkGray);
        colors.put("black", Color.black);
        if (delay>0){ // Creation du thread de synchronisation du GUI
            new Thread(new Runnable(){public void run(){
                while(true){
                    try{Thread.sleep(delay);}catch(Exception e){}
		    syncWithTopology();
		}
	    }}).start();
	}
    }
    public void setEditor(Editor editor){
        this.ed=editor;
    }
    public void selectVertex(Vertex v){
        if (this.ed!=null)
            this.ed.setSelectedVertex(v);
    }
    public void addJVertex(int x, int y){
        JVertex jv=new JVertex(this, t.addVertex(x, y));
        jvertices.add(jv);
        this.add(jv);
        this.updateUI();	
    }
    public void removeJVertex(JVertex jv){
        jvertices.remove(jv);
        t.removeVertex(jv.getVertex());
        this.remove(jv);
        this.updateUI();
    }
    public void paint(Graphics g){
        super.paint(g);
        for (int i=0; i<jvertices.size(); i++){
            Vertex vi = ((JVertex)jvertices.elementAt(i)).getVertex();
            for (int j=i; j<jvertices.size(); j++){
                Vertex vj = ((JVertex)jvertices.elementAt(j)).getVertex();
                if (t.communicate(vi, vj)){
                    if (vi!=vj){
                        g.setColor(Color.BLACK);
                        g.drawLine(vi.getX(),vi.getY(),vj.getX(),vj.getY());
                    }
                }
            }
        }
        for (int i=0; i<jvertices.size(); i++){
            Vertex vi = ((JVertex)jvertices.elementAt(i)).getVertex();
            String s =vi.getLabel("color");
            if (s!=null && s.compareTo("none") !=0){
                g.setColor((Color)colors.get(s));
                g.fillOval(vi.getX()-7, vi.getY()-7, 13, 13);
                g.setColor(Color.black);
            }
            for (int j=0; j<vi.neighbors.size(); j++){
                Neighbor n = ((Neighbor)vi.neighbors.elementAt(j));
                Vertex vj = n.getVertex();
                if (n.getLinkState()=="on")
                    if (n.getLinkLabel().compareTo("0")!=0)
                        drawLine(g,vi.getX(),vi.getY(),vj.getX(),vj.getY(),5);
            }
        }
    }
    public void mouseClicked(MouseEvent e){
    }
    public void mouseEntered(MouseEvent e){
    }
    public void mouseExited(MouseEvent e){
    }

    public void mousePressed(MouseEvent e) {
        if (e.getButton()==e.BUTTON1){
            addJVertex(e.getX(), e.getY());
        }
    }
    public void mouseReleased(MouseEvent e) {
    }

    // Cette methode permet de synchroniser l'interface graphique 
    // avec la topology sous-jacente
    public void syncWithTopology(){
        Vector vertices = t.getVertices();
        // enleve les JVertex dont le Vertex n'existe plus et replace les autres
        for (int jv=0; jv<jvertices.size(); jv++){
            JVertex jvertex=(JVertex)jvertices.elementAt(jv);
            boolean exist=false;
            for (int v=0; v<vertices.size(); v++){
                if (jvertex.getVertex()==(Vertex)vertices.elementAt(v)) 
                    exist=true;
            }
            if (exist){
                jvertex.move();
                jvertex.setToolTipText(jvertex.getVertex().toString());
            }else{
                jvertices.remove(jvertex);
                this.remove(jvertex);
            }
        }
        // ajoute un JVertex pour chaque nouveau Vertex trouve
        for (int v=0; v<vertices.size(); v++){
            boolean exist=false;
            for (int jv=0; jv<jvertices.size(); jv++){
                JVertex jvertex=(JVertex)jvertices.elementAt(jv);
                if (jvertex.getVertex()==(Vertex)vertices.elementAt(v)) 
                    exist=true;
            }
            if (!exist){
                JVertex jvertex=new JVertex(this, (Vertex)vertices.elementAt(v));
                jvertices.add(jvertex);
                this.add(jvertex);
            }
        }
        this.updateUI();
    }
    private static void drawLine(Graphics g,
                                int x1, int y1,
                                int x2, int y2,
                                int lineWidth) {
        if (lineWidth == 1)
            g.drawLine(x1, y1, x2, y2);
        else {
            double angle;
            double halfWidth = ((double)lineWidth)/2.0;
            double deltaX = (double)(x2 - x1);
            double deltaY = (double)(y2 - y1);
            if (x1 == x2)
                angle=Math.PI;
            else
                angle=Math.atan(deltaY/deltaX)+Math.PI/2;
            int xOffset = (int)(halfWidth*Math.cos(angle));
            int yOffset = (int)(halfWidth*Math.sin(angle));
            int[] xCorners = { x1-xOffset, x2-xOffset+1,
                               x2+xOffset+1, x1+xOffset };
            int[] yCorners = { y1-yOffset, y2-yOffset,
                               y2+yOffset+1, y1+yOffset+1 };
            g.fillPolygon(xCorners, yCorners, 4);
        }
    }
}
