// This file is part of Dagrs.

// Dagrs is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// Dagrs is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with Dagrs; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


import java.util.Vector;
import java.util.Hashtable;
import java.util.Random;
import java.awt.Point;

public class Topology{
    private double range;
    private Vector vertices;
    private Hashtable defaultLabel;
    private Random rand;
    private Recorder rec;
    public boolean exportLogic=false;
    
    public Topology(double range){
	this.range = range;
        this.rand = new Random();
	this.defaultLabel = new Hashtable();
	this.vertices = new Vector();
        this.rec = new Recorder(this);
    }
    public Topology(double range, Hashtable defaultLabel){
        this(range);
	this.defaultLabel = (Hashtable)defaultLabel.clone();
    }
    public void setDefaultLabel(Hashtable defaultLabel){
        this.defaultLabel = (Hashtable)defaultLabel.clone();
    }
    public Vector getVertices(){
	return vertices;
    }
    public double getRange(){
        return range;
    }
    public void setRange(double range){
        this.range = range;
    }
    public Vertex addVertex(int x, int y){
	Vertex tmp = new Vertex(defaultLabel, x, y);
        //tmp.setLabel("id", createID().toString());
	vertices.add(tmp);
	return tmp;
    }
    public Vertex addVertex(Hashtable labels, int x, int y){
	Vertex tmp = new Vertex(labels, x, y);
        //tmp.setLabel("id", createID().toString());
	vertices.add(tmp);
	return tmp;
    }
    public void removeVertex(Vertex v){
	vertices.remove(v);
    }
    public void removeAll(){
        vertices.clear();
    }
    public void snapEvolveStep(){
        rec.snapEvolveStep();
    }
    public Object getEvolve(){
        return rec;
    }
    public boolean communicate(Vertex v1, Vertex v2){
        if (v1!=v2)
            return (Vertex.distance(v1,v2)<range);
        else
            return false;
    }
    public void updateNeighbors(Vertex v1){
        int dist;
        v1.beginUpdate();
        for (int i=0; i<vertices.size(); i++){
            Vertex v2=(Vertex)vertices.elementAt(i);
            dist=(int)Vertex.distance(v1,v2);
            if (communicate(v1,v2)){//dist<range){
                v1.updateNeighbor(v2, dist);
            }
        }
        v1.endUpdate();
    }
    private Point getMinCoords(){
        int x=10000;
        int y=10000;
        for (int i=0; i<vertices.size(); i++){
            int xtmp=((Vertex)vertices.elementAt(i)).getX();
            int ytmp=((Vertex)vertices.elementAt(i)).getY();
            if (xtmp<x) x=xtmp;
            if (ytmp<y) y=ytmp;
        }
        return new Point(x,y);
    }
    private Point getMaxCoords(){
        int x=0;
        int y=0;
        for (int i=0; i<vertices.size(); i++){
            int xtmp=((Vertex)vertices.elementAt(i)).getX();
            int ytmp=((Vertex)vertices.elementAt(i)).getY();
            if (xtmp>x) x=xtmp;
            if (ytmp>y) y=ytmp;
        }
        return new Point(x,y);
    }
    // return the label of the edge from v1 to v2
    private String getLinkLabel(Vertex v1, Vertex v2){
        for (int i=0; i<v1.neighbors.size(); i++){
            Neighbor n = (Neighbor)v1.neighbors.elementAt(i);
            if (n.getVertex()==v2)
                return n.getLinkLabel();
        }
        return null;
    }
    public String toString(){
        String sf="", sv="", se="";
        Point pmin=getMinCoords();
        Point pmax=getMaxCoords();
        int xmin=(int)pmin.getX();
        int ymin=(int)pmin.getY();
        int xmax=(int)pmax.getX();
        int ymax=(int)pmax.getY();
        for(int i=0; i<vertices.size();i++){
            Vertex vi=(Vertex)vertices.elementAt(i);
            int xi=vi.getX()-xmin;
            int yi=vi.getY()-ymin;
            sv = sv + i +" "+xi+" "+yi+"\n";
            for(int j=i; j<vertices.size();j++){
                Vertex vj=(Vertex)vertices.elementAt(j);
                if (communicate(vi,vj)){
                    se = se + i + " " + j;
                    if (exportLogic)
                        se = se + " " + getLinkLabel(vi,vj);
                    se = se + "\n";
                }
            }
        }
        sf="dimensions\n";
        sf=sf+(xmax-xmin)+" "+(ymax-ymin)+"\n\n";
        sf=sf+"vertices\n";
        sf=sf+sv+"\n";
        sf=sf+"edges\n";
        sf=sf+se+"\n";
        return sf;
    }
    /*private synchronized Integer createID(){
        Integer id=new Integer(0);
        boolean ok=false;
        while (!ok){
            id=new Integer(rand.nextInt(1000));
            ok=true;
            for (int i=0; i<vertices.size(); i++){
                Vertex v=(Vertex)vertices.elementAt(i);
                if ((new Integer(v.getLabel("id"))).compareTo(id)==0)
                    ok=false;
            }
        }
        return id;
        }*/
}
