// This file is part of Dagrs.

// Dagrs is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// Dagrs is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with Dagrs; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


import java.util.*;

public class Vertex{
    private int x, y;
    private Hashtable labels;
    public Vector neighbors;
    public boolean busy=false;

    public Vertex(){
        labels = new Hashtable();
        neighbors = new Vector();
    }
    public Vertex(int x, int y){
	this();
	move(x, y);
    }
    public Vertex(Hashtable labels){
	this();
	this.labels = (Hashtable)labels.clone();
    }
    public Vertex(Hashtable labels, int x, int y){
	this(x, y);
	this.labels = (Hashtable)labels.clone();
    }

    public int getX(){return x;}
    public int getY(){return y;}
    public String getLabel(String key){
	return (String)labels.get(key);
    }
    public Hashtable getAllLabels(){
        return labels;
    }
    public void setLabel(String key, String value){
	labels.put(key, value);
    }
    public void setAllLabels(Hashtable labels){
	this.labels = (Hashtable)labels.clone();	
    }
    public void move(int x, int y){
	this.x = x;
	this.y = y;
    }
    public void beginUpdate(){
        for (int i=0; i<neighbors.size(); i++){
            ((Neighbor)neighbors.elementAt(i)).setUpdated(0);
        }
    }
    public void updateNeighbor(Vertex v, int dist){
        Neighbor n;
        int exist=0;
        for (int i=0; i<neighbors.size(); i++){
            n=(Neighbor)neighbors.elementAt(i);
            if ((Vertex)n.getVertex() == v){
                exist=1;
                n.setDist(dist);
                n.setUpdated(1);
                n.setLinkState("on");
            }
        }
        if (exist==0){
            n=(new Neighbor(v, "0", dist));
            n.setUpdated(1);
            neighbors.add(n);
        }
    }
    public void endUpdate(){
        // If a node has not been yet updated here 
        // then it is a disappearing node
        Neighbor n;
        for (int i=0; i<neighbors.size(); i++){
            n=(Neighbor)neighbors.elementAt(i);
            if (n.getUpdated()==0){ // stabilized => node removed
                if (n.getLinkLabel().compareTo("0")==0){
                    neighbors.remove(n);
                    i--;
                }
                else // set the node as disappeared
                    n.setLinkState("off");
            }
        }
    }
    public static double distance(Vertex v1, Vertex v2){
        double diffx = (double) Math.abs (v1.x - v2.x);
        double diffy = (double) Math.abs (v1.y - v2.y);
        return Math.sqrt(diffx*diffx+diffy*diffy);	
    }
    public String toString(){
	String allLabels="";
	for (Enumeration e = labels.elements(); e.hasMoreElements() ;) {
	    allLabels=allLabels + (String)e.nextElement() + ",";
	}
	if (allLabels=="") return super.toString();
	else return allLabels.substring(0,allLabels.length()-1);
    }
}
