import java.util.*;
import java.io.*;

public class EvolvingGraph{
    private Vector nodes;
    private Vector edges;
    private Integer maxTime=0;
    private Vector journeys;

    public EvolvingGraph(){
        nodes=new Vector();
        edges=new Vector();
        journeys=new Vector();
    }
    public EvolvingGraph(String filename){
        this();
        try{
            BufferedReader in=new BufferedReader(new FileReader(filename));
            StringTokenizer st;
            String sin="";
            do sin=in.readLine(); while (sin.compareTo("vertices")!=0);
            st = new StringTokenizer(sin," ");
            while ((sin=in.readLine()).compareTo("")!=0){
                st = new StringTokenizer(sin," ");
                String id=st.nextToken();
                addNode(id);
            }

            do sin=in.readLine(); while (sin.compareTo("edges")!=0);
            while ((sin=in.readLine()).compareTo("")!=0){
                st = new StringTokenizer(sin," ");
                String times="";
                String n1=st.nextToken();
                String n2=st.nextToken();
                
                // Get the time label
                if (st.hasMoreTokens())
                    times=st.nextToken();
                while (st.hasMoreTokens())
                    times=times+" "+st.nextToken();

                addEdge(n1,n2,intervalFromString(times));
            }
        }catch(Exception e){
        }        
    }

    class Node{
        public String id;
        public Node(String id){
            this.id=id.toString();
        }
    }
    class Edge{
        public String n1;
        public String n2;
        public Vector times;

        public Edge(String n1, String n2){
            this.n1=n1.toString();
            this.n2=n2.toString();
            this.times=new Vector();
        }
        public Edge(String n1, String n2, Integer time){
            this(n1, n2);
            this.times.add(time);
        }
        public Edge(String n1, String n2, Vector times){
            this(n1, n2);
            this.times=times;
        }
        public boolean equal(String n1, String n2){
            return ((this.n1.compareTo(n1)==0 && this.n2.compareTo(n2)==0) ||
                    (this.n1.compareTo(n2)==0 && this.n2.compareTo(n1)==0));
        }
    }

    public void addNode(String id){
        boolean exists=false;
        for (int i=0; i<nodes.size(); i++)
            if (((Node)nodes.elementAt(i)).id.compareTo(id)==0)
                exists=true;
        if (!exists)
            nodes.add(new Node(id));
    }

    public void addEdge(String n1, String n2, Vector times){
        Edge e;
        boolean exists=false;
        Vector v=times;
        for (int i=0; i<edges.size(); i++){
            e=(Edge)edges.elementAt(i);
            if (e.equal(n1,n2)){
                exists=true;
                e.times=v;
            }
        }
        if (!exists)
            edges.add(new Edge(n1,n2,v));
        if (((Integer)v.elementAt(v.size()-1))>maxTime)
            maxTime=(Integer)v.elementAt(v.size()-1);
    }

    public static String intervalToString(Vector vtimes){
        String stimes=" ";
        for (int i=0; i<vtimes.size()-1; i++){
            Integer t=(Integer)vtimes.elementAt(i);
            char last=stimes.charAt(stimes.length()-1);
            if (last==' '){
                stimes=stimes+t.toString();
                if (((Integer)vtimes.elementAt(i+1)).compareTo(t+1)==0)
                    stimes=stimes+"-";
                else
                    stimes=stimes+" ";
            }else{
                if (((Integer)vtimes.elementAt(i+1)).compareTo(t+1)!=0)
                    stimes=stimes+t.toString()+" ";
            }
        }
        stimes=stimes+vtimes.elementAt(vtimes.size()-1);
        stimes=stimes.substring(1, stimes.length());
        return stimes;
    }
    public static Vector intervalFromString(String stimes){
        StringTokenizer st=new StringTokenizer(stimes);
        Vector vtimes=new Vector();
        while (st.hasMoreElements()){
            String tmp=st.nextToken();
            int ind=tmp.indexOf('-');
            if (ind==-1){
                vtimes.add(new Integer(tmp));
            }else{
                int beg=Integer.parseInt(tmp.substring(0,ind));
                int end=Integer.parseInt(tmp.substring(ind+1,tmp.length()));
                for (int i=beg; i<=end; i++){
                    vtimes.add(new Integer(i));
                }
            }
        }
        return vtimes;
    }
    public String toString(){
        String sv="", se="";
        int id;
        for (int i=0; i<nodes.size(); i++){
            Node n=(Node)nodes.elementAt(i);
            sv=sv+n.id+"\n";
        }
        for (int i=0; i<edges.size(); i++){
            Edge e=(Edge)edges.elementAt(i);
            se=se+e.n1+" "+e.n2+" "+intervalToString(e.times)+"\n";
        }
        String sf="\nvertices\n";
        sf=sf+sv+"\n";
        sf=sf+"edges\n";
        sf=sf+se;
        return sf;
    }

    ///////////// METHODES /////////////////////
    
    public Vector getNeighbors(String n){
        Vector neighbors=new Vector();
        for (int i=0; i<edges.size(); i++){
            Edge e=(Edge)edges.elementAt(i);
            if (e.n1.compareTo(n)==0)
                neighbors.add(e.n2);
            else if (e.n2.compareTo(n)==0)
                neighbors.add(e.n1);
        }
        return neighbors;
    }
    public EvolvingGraph getTimeCut(Integer beg, Integer end){
        EvolvingGraph newEG=new EvolvingGraph();
        for (int i=0; i<nodes.size(); i++){
            Node n=(Node)nodes.elementAt(i);
            newEG.addNode(n.id);
        }
        for (int i=0; i<edges.size(); i++){
            Edge e=(Edge)edges.elementAt(i);
            Vector times=new Vector();
            for (int j=0; j<e.times.size(); j++){
                Integer t=(Integer)e.times.elementAt(j);
                if (t>=beg && t<end)
                    times.add(t);
            }
            if (times.size()>0){
                newEG.addEdge(e.n1, e.n2, times);
            }
        }
        return newEG;
    }
    
    public void buildJourneys(){
        for (int i=0; i<nodes.size(); i++){
            Vector destinations=new Vector(nodes.size());
            destinations.setElementAt(new Integer(0),0);
            for (int j=1; j<destinations.size(); j++)
                destinations.setElementAt(new Integer(Integer.MAX_VALUE),j);
            //Vector edges
        }
    }
    public boolean existsJourney(Node n1, Node n2, Vector visitedNodes){
        return true;//visitedNodes.
    }
}