import java.util.*;
import java.io.*;

public class EvolvingGraph{
    private Hashtable nodes;
    private Vector edges;

    public EvolvingGraph(){
        nodes=new Hashtable();
        edges=new Vector();
    }
    public EvolvingGraph addNode(Node n){
        nodes.put(n.getId(),n);
        return this;
    }
    public EvolvingGraph removeNode(Node n){
        Vector nedges=n.getEdges();
        for (int i=0; i<nedges.size(); i++)
            this.removeEdge((Edge)nedges.elementAt(i));
        nodes.remove(n.getId());
        return this;
    }
    public Node getNodeById(String id){
        return (Node)nodes.get(id);
    }
    public Vector getNodesIds(){
        Vector ret=new Vector();
        for (Enumeration enu=nodes.elements(); enu.hasMoreElements(); )
            ret.add(((Node)enu.nextElement()).getId());
        return ret;
    }
    public EvolvingGraph addEdge(Edge e){
        Vector epts=e.getEndpoints();
        ((Node)epts.elementAt(0)).addEdge(e);
        ((Node)epts.elementAt(1)).addEdge(e);
        edges.add(e);
        return this;
    }
    public EvolvingGraph removeEdge(Edge e){
        Vector epts=e.getEndpoints();
        ((Node)epts.elementAt(0)).removeEdge(e);
        ((Node)epts.elementAt(1)).removeEdge(e);
        edges.remove(e);
        return this;
    }
    public Edge getEdge(Node n1, Node n2){
        Vector n1edges=n1.getEdges();
        for (int i=0; i<n1edges.size(); i++){
            Edge e=(Edge)n1edges.elementAt(i);
            if (e.getOtherEndpoint(n1)==n2)
                return e;
        }
        return null;
    }
    public EvolvingGraph copy(){
        EvolvingGraph newEG=new EvolvingGraph();

        for (Enumeration enu=nodes.elements(); enu.hasMoreElements(); )
            newEG.addNode(new Node(((Node)enu.nextElement()).getId()));
        
        for (int i=0; i<edges.size(); i++){
            Edge e=(Edge)edges.elementAt(i);
            Vector epts=e.getEndpoints();
            Node n1=newEG.getNodeById(((Node)epts.elementAt(0)).getId());
            Node n2=newEG.getNodeById(((Node)epts.elementAt(1)).getId());
            Edge ne=new Edge(n1, n2, e.getTimes());
            newEG.addEdge(ne);
        }
        return newEG;
    }

    public EvolvingGraph doTimeCut(Integer beg, Integer end){
        for (int i=0; i<edges.size(); i++){
            Edge e=(Edge)edges.elementAt(i);
            if (e.doTimeCut(beg, end)==null){
                this.removeEdge(e);
                i--;
            }
        }
        return this;
    }

    public String toString(){
        String sv="", se="";
        for (Enumeration enu=nodes.elements(); enu.hasMoreElements(); ){
            Node n=(Node)enu.nextElement();
            sv=sv+n.getId()+" -> ";
            Vector neig=n.getNeighbors();
            for (int i=0; i<neig.size(); i++)
                sv=sv+((Node)neig.elementAt(i)).getId()+",";
            sv=sv+"\n";
        }
        for (int i=0; i<edges.size(); i++){
            Edge e=(Edge)edges.elementAt(i);
            Vector epts=e.getEndpoints();
            Node n1=(Node)epts.elementAt(0);
            Node n2=(Node)epts.elementAt(1);
            se=se+n1.getId()+" "+n2.getId()+" "+e.timesAsString()+"\n";
        }
        String sf="\nvertices\n";
        sf=sf+sv+"\n";
        sf=sf+"edges\n";
        sf=sf+se;
        return sf;
    }
}

